<?php

/**
 * @file
 * Admin callbacks and functions.
 */

/**
 * Admin settings form.
 */
function loggable_admin_settings() {
  $form = array();
  $settings = loggable_get_connection_settings();
  $form['connection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection'),
  );
  $form['connection']['loggable_api_key'] = array(
    '#type' => 'textfield',
    '#title' => t('API key'),
    '#description' => t('Enter your Loggable API key. This can be found <a href="@url">here</a>.', ['@url' => $settings['domain'] . '/key']),
    '#default_value' => $settings['api_key'],
    '#required' => TRUE,
  );
  $form['connection']['loggable_channel_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Channel ID'),
    '#description' => t('Enter the ID of the channel you wish to store events in. This can be found <a href="@url">here</a>.', ['@url' => $settings['domain'] . '/key']),
    '#default_value' => $settings['channel_id'],
    '#required' => TRUE,
  );
  $form['connection']['loggable_domain'] = array(
    '#type' => 'textfield',
    '#title' => t('Domain'),
    '#description' => t('Enter the domain where your Loggable instance is located. This should not change unless you are instructed.'),
    '#default_value' => $settings['domain'],
    '#required' => TRUE,
  );
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filters'),
    '#description' => t('Events must match at least one filter in order to be sent to Loggable.'),
    '#tree' => TRUE,
  );
  $form['filters']['add_new'] = array(
    '#type' => 'fieldset',
    '#title' => '<em>' . t('Add new filter') . '</em>',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    'form' => _loggable_filter_form(),
  );

  // Iterate the filters.
  foreach (loggable_get_filters() as $filter) {
    $form['filters'][] = array(
      '#type' => 'fieldset',
      '#title' => $filter['label'],
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'form' => _loggable_filter_form($filter),
    );
  }

  return system_settings_form($form);
}

/**
 * Validation handler for the admin settings form.
 */
function loggable_admin_settings_validate($form, &$form_state) {
  $filters = array();

  // Iterate the filters in the form.
  foreach ($form_state['values']['filters'] as $key => $values) {
    // Extract the filter.
    $filter = $values['form'];

    // Check if this filter should be deleted.
    if ($filter['delete']) {
      continue;
    }

    // Remove the delete value.
    unset($filter['delete']);

    // Convert the types to an array.
    $filter['types'] = array_filter(array_map('trim', explode("\n", $filter['types'])));

    // Clean the levels.
    $filter['levels'] = array_values($filter['levels']);

    foreach ($filter['levels'] as $index => $value) {
      if (!is_string($value)) {
        unset($filter['levels'][$index]);
      }
      else {
        $filter['levels'][$index] = (int) $value;
      }
    }

    // Check if this is the new form.
    if ($key === 'add_new') {
      // Check if any value are present.
      if ($filter['label'] || $filter['levels'] || $filter['types']) {
        // Check that a label is provided.
        if (!$filter['label']) {
          form_set_error('filters][add_new][form][label', t('New filters require a label.'));
        }

        // Check that levels are provided.
        if (!$filter['levels']) {
          form_set_error('filters][add_new][form][levels', t('New filters require at least one severity level.'));
        }
      }
      else {
        continue;
      }
    }

    // Store the filter.
    $filters[] = $filter;
  }

  // Store the filters so they are saved.
  $form_state['values']['loggable_filters'] = $filters;
}

/**
 * Provide a form for a filter.
 *
 * @param array $filter
 *   A filter array.
 *
 * @return array
 *   A form array.
 */
function _loggable_filter_form(array $filter = array()) {
  // Determine if this is for a new filter.
  $new = empty($filter);

  // Provide a default filter, if needed.
  if (!$filter) {
    $filter = array(
      'label' => NULL,
      'levels' => array(),
      'types' => array(),
      'enabled' => TRUE,
    );
  }

  $form = array();
  $form['label'] = array(
    '#type' => 'textfield',
    '#title' => t('Label'),
    '#description' => t('Add an identifier for this filter.'),
    '#default_value' => $filter['label'],
    '#required' => !$new,
  );
  $form['levels'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Severity levels'),
    '#description' => t('Select the severity levels.'),
    '#options' => array(
      WATCHDOG_ALERT => t('Alert'),
      WATCHDOG_CRITICAL => t('Critical'),
      WATCHDOG_DEBUG => t('Debug'),
      WATCHDOG_EMERGENCY => t('Emergency'),
      WATCHDOG_ERROR => t('Error'),
      WATCHDOG_INFO => t('Info'),
      WATCHDOG_NOTICE => t('Notice'),
      WATCHDOG_WARNING => t('Warning'),
    ),
    '#default_value' => $filter['levels'],
    '#required' => !$new,
  );
  $form['types'] = array(
    '#type' => 'textarea',
    '#title' => t('Types'),
    '#description' => t('Optionally filter by the event type. You may enter an unlimited amount of types. Each type must go on a new line. Wildcard (*) characters are supported.'),
    '#default_value' => implode("\n", $filter['types']),
  );
  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#description' => t('If disabled, this filter will be ignored.'),
    '#default_value' => $filter['enabled'],
  );
  $form['delete'] = array(
    '#type' => 'checkbox',
    '#title' => t('Delete this filter'),
    '#description' => t('If checked, this filter will be deleted.'),
    '#default_value' => FALSE,
    '#access' => !$new,
  );
  return $form;
}
