<?php

/**
 * @file
 * Functions that provide a watchdog handler.
 */

/**
 * The request timeout length, in seconds.
 *
 * @var int
 */
const LOGGABLE_REQUEST_TIMEOUT = 3;

/**
 * The message max length.
 *
 * @var int
 */
const LOGGABLE_MESSAGE_MAXLENGTH = 5000;

/**
 * The generic field max length.
 *
 * @var int
 */
const LOGGABLE_FIELD_MAXLENGTH = 64;

/**
 * The amount of async requests to make at a time.
 *
 * @var int
 */
const LOGGABLE_ASYNC_MAX_REQUESTS = 5;

/**
 * Wrapper for hook_watchdog().
 */
function _loggable_watchdog(array $log_entry) {
  // Stop if the connection configuration is missing.
  if (!loggable_is_connection_configured()) {
    return;
  }

  // Load helper functions, if needed.
  if (!function_exists('drupal_substr')) {
    require_once DRUPAL_ROOT . '/includes/unicode.inc';
  }

  // Check if this event matches a filter.
  if (!loggable_filter_match($log_entry['severity'], $log_entry['type'])) {
    return;
  }

  // Format the variables, if needed.
  if (!is_array($log_entry['variables'])) {
    $log_entry['variables'] = array();
  }

  // Load the user, if one.
  $user = $log_entry['uid'] ? user_load($log_entry['uid']) : NULL;

  // Map the severity levels.
  $levels = array(
    WATCHDOG_ALERT => 'alert',
    WATCHDOG_CRITICAL => 'critical',
    WATCHDOG_DEBUG => 'debug',
    WATCHDOG_EMERGENCY => 'emergency',
    WATCHDOG_ERROR => 'error',
    WATCHDOG_INFO => 'info',
    WATCHDOG_NOTICE => 'notice',
    WATCHDOG_WARNING => 'warning',
  );

  // Generate the event data.
  $data = array(
    'user' => $user ? $user->name : NULL,
    'type' => drupal_substr($log_entry['type'], 0, LOGGABLE_FIELD_MAXLENGTH),
    'message' => drupal_substr(strtr($log_entry['message'], $log_entry['variables']), 0, LOGGABLE_MESSAGE_MAXLENGTH),
    'severity' => $levels[$log_entry['severity']],
    'url' => $log_entry['request_uri'],
    'created' => $log_entry['timestamp'],
  );

  // Queue the request.
  loggable_queue_request($data);
}

/**
 * Check if a given severity level and type match a filter.
 *
 * @param int $level
 *   The event severity level.
 * @param string $type
 *   The event type.
 *
 * @return bool
 *   TRUE if the event matches a filter, otherwise FALSE.
 */
function loggable_filter_match($level, $type) {
  // Iterate the filters.
  foreach (loggable_get_filters() as $filter) {
    // Check if the filter is enabled.
    if ($filter['enabled']) {
      // Check if the severity matches.
      if (in_array($level, $filter['levels'])) {
        // Check if a type filter is present.
        if ($filter['types']) {
          // Check for a type match.
          if (drupal_match_path($type, implode("\n", $filter['types']))) {
            return TRUE;
          }
          else {
            continue;
          }
        }

        return TRUE;
      }
    }
  }

  return FALSE;
}

/**
 * Queue a request to Loggable to send an event.
 *
 * @param array $data
 *   An array of event data.
 */
function loggable_queue_request(array $data) {
  $requests = &drupal_static(__FUNCTION__, []);

  // Check if this is the first incoming request.
  if (empty($requests)) {
    // Register the shutdown function to execute the request.
    drupal_register_shutdown_function('loggable_execute_queued_requests');
  }

  // Load the connection settings.
  $settings = loggable_get_connection_settings();

  // Build the payload.
  $payload = array(
    'data' => array(
      'type' => 'event',
      'attributes' => $data,
      'relationships' => array(
        'channel' => array(
          'data' => array(
            'type' => 'channel',
            'id' => $settings['channel_id'],
          ),
        ),
      ),
    ),
  );

  // Build the request.
  $request = curl_init();
  curl_setopt($request, CURLOPT_CUSTOMREQUEST, 'POST');
  curl_setopt($request, CURLOPT_URL, $settings['domain'] . '/api/event');
  curl_setopt($request, CURLOPT_HEADER, 1);
  curl_setopt($request, CURLOPT_RETURNTRANSFER, 0);
  curl_setopt($request, CURLOPT_POSTFIELDS, drupal_json_encode($payload));
  curl_setopt($request, CURLOPT_HTTPHEADER, array(
    'Content-Type: application/vnd.api+json',
    'Accept: application/vnd.api+json',
    'api-key: ' . $settings['api_key'],
  ));

  // Store the request.
  $requests[] = $request;
}

/**
 * Execute the queued requests.
 */
function loggable_execute_queued_requests() {
  $requests = &drupal_static('loggable_queue_request', []);
  $requests = array_chunk(array_reverse($requests), LOGGABLE_ASYNC_MAX_REQUESTS);

  // Initialize a curl_multi object.
  $curl = curl_multi_init();

  // Iterate the requests.
  foreach ($requests as $batch) {
    // Iterate the batch.
    foreach ($batch as $request) {
      // Add the request.
      curl_multi_add_handle($curl, $request);
    }

    // Execute the requests.
    do {
      do {
        $multi_result = curl_multi_exec($curl, $active);
      } while ($multi_result == CURLM_CALL_MULTI_PERFORM);

      // Block until there is activity on any of the handlers. Avoids
      // busywaiting.
      if ($multi_result == CURLM_OK) {
        // According to https://bugs.php.net/bug.php?id=63411, in PHP 5.3.18+
        // curl_multi_select can return immediately if libcurl cannot block, but
        // still has something to process, so we block a bit ourselves.
        if (curl_multi_select($curl) == -1) {
          // Wait 0.1 second.
          usleep(100000);
        }
      }
    } while ($active && $multi_result == CURLM_OK);
  }

  // Close cURL.
  curl_multi_close($curl);
}
